@@ -1,7 +1,11 @@
 # :coding: utf-8
 # :copyright: Copyright (c) 2015 ftrack
 
+from builtins import zip
+from six import string_types
+from builtins import object
 import collections
+from six.moves import collections_abc
 import functools
 
 import ftrack_api.entity.base
@@ -11,6 +15,15 @@ import ftrack_api.symbol
 import ftrack_api.inspection
 from ftrack_api.logging import LazyLogMessage as L
 
+from future.utils import with_metaclass
+
+
+MixinBaseClass = with_metaclass(
+      ftrack_api.entity.base.DynamicEntityTypeMetaclass,
+      ftrack_api.entity.base._EntityBase,
+      collections_abc.MutableMapping
+)
+
 
 class Location(ftrack_api.entity.base.Entity):
     '''Represent storage for components.'''
@@ -121,8 +134,8 @@ class Location(ftrack_api.entity.base.Entity):
 
         '''
         if (
-            isinstance(sources, basestring)
-            or not isinstance(sources, collections.Sequence)
+            isinstance(sources, string_types)
+            or not isinstance(sources, collections_abc.Sequence)
         ):
             sources = [sources]
 
@@ -181,7 +194,7 @@ class Location(ftrack_api.entity.base.Entity):
                     source = sources[index]
 
                 # Add members first for container components.
-                is_container = 'members' in component.keys()
+                is_container = 'members' in list(component.keys())
                 if is_container and recursive:
                     self.add_components(
                         component['members'], source, recursive=recursive,
@@ -254,10 +267,10 @@ class Location(ftrack_api.entity.base.Entity):
         # Publish events.
         for component in components_to_register:
 
-            component_id = ftrack_api.inspection.primary_key(
+            component_id = list(ftrack_api.inspection.primary_key(
                 component
-            ).values()[0]
-            location_id = ftrack_api.inspection.primary_key(self).values()[0]
+            ).values())[0]
+            location_id = list(ftrack_api.inspection.primary_key(self).values())[0]
 
             self.session.event_hub.publish(
                 ftrack_api.event.base.Event(
@@ -313,7 +326,7 @@ class Location(ftrack_api.entity.base.Entity):
                 details=dict(location=self)
             )
 
-        is_container = 'members' in component.keys()
+        is_container = 'members' in list(component.keys())
         if is_container:
             # TODO: Improve this check. Possibly introduce an inspection
             # such as ftrack_api.inspection.is_sequence_component.
@@ -363,7 +376,7 @@ class Location(ftrack_api.entity.base.Entity):
             chunked_read = functools.partial(
                 source_data.read, ftrack_api.symbol.CHUNK_SIZE
             )
-            for chunk in iter(chunked_read, ''):
+            for chunk in iter(chunked_read, b''):
                 target_data.write(chunk)
 
             target_data.close()
@@ -421,7 +434,7 @@ class Location(ftrack_api.entity.base.Entity):
             self.get_resource_identifier(component)
 
             # Remove members first for container components.
-            is_container = 'members' in component.keys()
+            is_container = 'members' in list(component.keys())
             if is_container and recursive:
                 self.remove_components(
                     component['members'], recursive=recursive
@@ -434,10 +447,10 @@ class Location(ftrack_api.entity.base.Entity):
             self._deregister_component_in_location(component)
 
             # Emit event.
-            component_id = ftrack_api.inspection.primary_key(
+            component_id = list(ftrack_api.inspection.primary_key(
                 component
-            ).values()[0]
-            location_id = ftrack_api.inspection.primary_key(self).values()[0]
+            ).values())[0]
+            location_id = list(ftrack_api.inspection.primary_key(self).values())[0]
             self.session.event_hub.publish(
                 ftrack_api.event.base.Event(
                     topic=ftrack_api.symbol.COMPONENT_REMOVED_FROM_LOCATION_TOPIC,
@@ -470,8 +483,8 @@ class Location(ftrack_api.entity.base.Entity):
 
     def _deregister_component_in_location(self, component):
         '''Deregister *component* from location.'''
-        component_id = ftrack_api.inspection.primary_key(component).values()[0]
-        location_id = ftrack_api.inspection.primary_key(self).values()[0]
+        component_id = list(ftrack_api.inspection.primary_key(component).values())[0]
+        location_id = list(ftrack_api.inspection.primary_key(self).values())[0]
 
         # TODO: Use session.get for optimisation.
         component_location = self.session.query(
@@ -526,7 +539,8 @@ class Location(ftrack_api.entity.base.Entity):
             for index, resource_identifier in enumerate(resource_identifiers):
                 resource_identifiers[index] = (
                     self.resource_identifier_transformer.decode(
-                        resource_identifier
+                        resource_identifier,
+                        context={'component': components[index]}
                     )
                 )
 
@@ -541,17 +555,17 @@ class Location(ftrack_api.entity.base.Entity):
         '''
         component_ids_mapping = collections.OrderedDict()
         for component in components:
-            component_id = ftrack_api.inspection.primary_key(
+            component_id = list(ftrack_api.inspection.primary_key(
                 component
-            ).values()[0]
+            ).values())[0]
             component_ids_mapping[component_id] = component
 
         component_locations = self.session.query(
             'select component_id, resource_identifier from ComponentLocation '
             'where location_id is {0} and component_id in ({1})'
             .format(
-                ftrack_api.inspection.primary_key(self).values()[0],
-                ', '.join(component_ids_mapping.keys())
+                list(ftrack_api.inspection.primary_key(self).values())[0],
+                ', '.join(list(component_ids_mapping.keys()))
             )
         )
 
@@ -563,7 +577,7 @@ class Location(ftrack_api.entity.base.Entity):
 
         resource_identifiers = []
         missing = []
-        for component_id, component in component_ids_mapping.items():
+        for component_id, component in list(component_ids_mapping.items()):
             if component_id not in resource_identifiers_map:
                 missing.append(component)
             else:
@@ -607,7 +621,7 @@ class Location(ftrack_api.entity.base.Entity):
         return self.accessor.get_url(resource_identifier)
 
 
-class MemoryLocationMixin(object):
+class MemoryLocationMixin(MixinBaseClass):
     '''Represent storage for components.
 
     Unlike a standard location, only store metadata for components in this
@@ -627,7 +641,7 @@ class MemoryLocationMixin(object):
 
     def _register_component_in_location(self, component, resource_identifier):
         '''Register *component* in location with *resource_identifier*.'''
-        component_id = ftrack_api.inspection.primary_key(component).values()[0]
+        component_id = list(ftrack_api.inspection.primary_key(component).values())[0]
         self._cache[component_id] = resource_identifier
 
     def _register_components_in_location(
@@ -645,7 +659,7 @@ class MemoryLocationMixin(object):
 
     def _deregister_component_in_location(self, component):
         '''Deregister *component* in location.'''
-        component_id = ftrack_api.inspection.primary_key(component).values()[0]
+        component_id = list(ftrack_api.inspection.primary_key(component).values())[0]
         self._cache.pop(component_id)
 
     def _get_resource_identifiers(self, components):
@@ -658,9 +672,9 @@ class MemoryLocationMixin(object):
         resource_identifiers = []
         missing = []
         for component in components:
-            component_id = ftrack_api.inspection.primary_key(
+            component_id = list(ftrack_api.inspection.primary_key(
                 component
-            ).values()[0]
+            ).values())[0]
             resource_identifier = self._cache.get(component_id)
             if resource_identifier is None:
                 missing.append(component)
@@ -675,7 +689,7 @@ class MemoryLocationMixin(object):
         return resource_identifiers
 
 
-class UnmanagedLocationMixin(object):
+class UnmanagedLocationMixin(MixinBaseClass):
     '''Location that does not manage data.'''
 
     def _add_data(self, component, resource_identifier, source):
@@ -712,7 +726,7 @@ class OriginLocationMixin(MemoryLocationMixin, UnmanagedLocationMixin):
         return context
 
 
-class ServerLocationMixin(object):
+class ServerLocationMixin(MixinBaseClass):
     '''Location representing ftrack server.
 
     Adds convenience methods to location, specific to ftrack server.
